/**
 * Copyright 2010 OpenEngSB Division, Vienna University of Technology
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.openengsb.core.workflow.internal;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.openengsb.core.common.workflow.RuleBaseException;
import org.openengsb.core.common.workflow.RuleManager;
import org.openengsb.core.common.workflow.model.RuleBaseElementId;
import org.openengsb.core.common.workflow.model.RuleBaseElementType;

public abstract class AbstractRuleManager implements RuleManager {

    protected ResourceHandler<?> getRessourceHandler(RuleBaseElementType element) {
        throw new UnsupportedOperationException("not implemented for type " + getClass());
    }

    @Override
    public void add(RuleBaseElementId name, String code) throws RuleBaseException {
        this.getRessourceHandler(name.getType()).create(name, code);
    }

    @Override
    public String get(RuleBaseElementId name) {
        return this.getRessourceHandler(name.getType()).get(name);
    }

    @Override
    public void update(RuleBaseElementId name, String newCode) throws RuleBaseException {
        this.getRessourceHandler(name.getType()).update(name, newCode);
    }

    @Override
    public void delete(RuleBaseElementId name) throws RuleBaseException {
        this.getRessourceHandler(name.getType()).delete(name);
    }

    @Override
    public Collection<RuleBaseElementId> list(RuleBaseElementType type) {
        return this.getRessourceHandler(type).list();
    }

    @Override
    public Collection<RuleBaseElementId> list(RuleBaseElementType type, String packageName) {
        return this.getRessourceHandler(type).list(packageName);
    }

    @Override
    public void addGlobalIfNotPresent(String className, String name) throws RuleBaseException {
        String currentType = listGlobals().get(name);
        if (currentType == null) {
            addGlobal(className, name);
            return;
        }
        if (!currentType.equals(className)) {
            throw new IllegalArgumentException(
                String
                    .format(
                        "Unable to add global of type %s."
                                + "Global with the same name but different type is already registered (%s)",
                        className, currentType));
        }
    }

    @Override
    public void addOrUpdate(RuleBaseElementId name, String code) throws RuleBaseException {
        if (get(name) == null) {
            add(name, code);
        } else {
            update(name, code);
        }
    }

    @Override
    public Collection<String> getAllGlobalsOfType(String type) {
        Set<String> result = new HashSet<String>();
        for (Map.Entry<String, String> entry : listGlobals().entrySet()) {
            if (entry.getValue().equals(type)) {
                result.add(entry.getKey());
            }
        }
        return result;
    }

    @Override
    public String getGlobalType(String name) {
        return this.listGlobals().get(name);
    }
}
